package com.persagy.dmp.rwd.delivery.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.map.MapUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.dmp.common.constant.ValidEnum;
import com.persagy.dmp.common.model.entity.BaseEntity;
import com.persagy.dmp.delivery.entity.RelationDefineCollect;
import com.persagy.dmp.rwd.delivery.dao.RelationDefineCollectMapper;
import com.persagy.dmp.rwd.delivery.service.IRelationDefineCollectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * 项目，现场交付范围管理-关系
 * @author:linhuili
 * @date:2021/10/13
 */
@Service
public class RelationDefineCollectServiceImpl extends ServiceImpl<RelationDefineCollectMapper, RelationDefineCollect> implements IRelationDefineCollectService {

    @Autowired
    private RelationDefineCollectMapper relationDefineCollectMapper;

    /**
     * 数量统计
     * @param queryWrapper 查询条件
     * @return
     */
    @Override
    public Integer queryCount(Wrapper<RelationDefineCollect> queryWrapper) {
        return relationDefineCollectMapper.selectCount(queryWrapper);
    }

    /**
     * 查询信息点交付范围
     * @param queryWrapper 查询条件
     * @return
     */
    @Override
    public List<RelationDefineCollect> queryByCondition(Wrapper<RelationDefineCollect> queryWrapper) {
        return relationDefineCollectMapper.selectList(queryWrapper);
    }

    /**
     * 分页询信息点交付范围
     * @param page 分页条件
     * @param queryWrapper 查询条件
     * @return
     */
    @Override
    public Page queryByCondition(Page page, Wrapper<RelationDefineCollect> queryWrapper) {
        return relationDefineCollectMapper.selectPage(page, queryWrapper);
    }

    /**
     * 新增信息点交付范围
     * @param voList 待新增数据
     * @return
     */
    @Override
    public List<RelationDefineCollect> insert(List<RelationDefineCollect> voList) throws Exception{
        //新增采集信息点
        saveOrUpdateBatch(voList);
        return voList;
    }

    /**
     * 修改信息点交付范围
     * @param voList 待修改数据
     * @return
     */
    @Override
    public List<RelationDefineCollect> update(List<RelationDefineCollect> voList) throws Exception {
        //修改采集信息点
        saveOrUpdateBatch(voList);
        return voList;
    }

    /**
     * 删除信息点交付范围
     * @param idList 主键
     */
    @Override
    public void delete(List<String> idList) {
        if(CollUtil.isEmpty(idList)) {
            return;
        }
        List<RelationDefineCollect> dbList = relationDefineCollectMapper.selectBatchIds(idList);
        Map<String, RelationDefineCollect> dbMap = CollectionUtil.fieldValueMap(dbList, BaseEntity.PROP_ID);
        idList.forEach(id -> {
            RelationDefineCollect dbVO = MapUtil.get(dbMap, id, RelationDefineCollect.class);
            // 容错处理
            if(dbVO == null) {
                return;
            }
            // 删除
            RelationDefineCollect vo = new RelationDefineCollect();
            vo.setId(id);
            vo.setValid(ValidEnum.FALSE.getType());
            relationDefineCollectMapper.updateById(vo);
        });
    }
}
